//  
//  Copyright (C) 2011 Ergo Developers
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using Gtk;
using Ergo;
using Ergo.Widgets;
using Sqlite;

namespace Ergo.Backend {

	public class ErgoSqlBackend : GLib.Object {
	
        private Database db;
        private Statement stmt;
        private ErgoWindow window;
        
        
/*
Database: data/ergo.db        
Tables:
    * ergo:
        task            STRING
        notes           STRING
        date            INT
        done            INT
        done_date       INT
        task_list_id    INT
        
    * task_lists:
        name            STRING
        tasks           INT
        tasks_done      INT
*/        
	
	    public ErgoSqlBackend(ErgoWindow window){
	        this.window = window;
	        if (!FileUtils.test (Environment.get_home_dir () + "/.local/share/ergo/data.db", FileTest.IS_REGULAR)) {
	            
                print("Need to make a database at the specific place!\n");
                DirUtils.create(Environment.get_home_dir () + "/.local/share/ergo/", 0777);
       	        Database.open_v2(Environment.get_home_dir () + "/.local/share/ergo/data.db", out this.db, Sqlite.OPEN_READWRITE | Sqlite.OPEN_CREATE,null);
       	        init();
                // Everything goes when i have a database installed at bin or build - at the place i launch ergo.
                // Need to update the fields to the ultimate path to the database
	        }
   	        Database.open_v2(Environment.get_home_dir () + "/.local/share/ergo/data.db", out this.db, OPEN_READWRITE);
//            if (new_table) {init();}
	    }	    
	    
	    // Create the table on first launch
	    public void init(){
    	    this.db.exec("create table ergo (task STRING, notes TEXT, date INT, done INT, date_done INT, task_list_id INT)");
    	    this.db.exec("create table task_lists (name STRING, tasks INT, done INT)");    	    
	    }
	    
	    
	    // Adds TaskList
	    public void add_task_list(string name){
            this.db.prepare_v2(@"insert into task_lists VALUES('$name', 0, 0)", -1, out this.stmt, null);
            this.stmt.step();
	    }
	    
	    // Remove Tasklist
	    public void remove_task_list(int id){
            this.db.prepare_v2(@"delete from task_list where ROWID = '$id'", -1, out this.stmt, null);
            this.stmt.step();    
	    }
	    
	    // Update Tasklist
        public void update_task_list(int id, string col, string newval) {
            this.db.prepare_v2(@"update task_list set $col = '$newval' where ROWID = '$id'", -1, out this.stmt, null);
            this.stmt.step();
        }	    
	    
	    // Removes a single task
        public void remove_task(int id) {
            this.db.prepare_v2(@"delete from ergo where ROWID = '$id'",-1, out this.stmt, null);
            this.stmt.step();    
            // update_task_list();
        }
        
        
// -----------------------------------------------------------------        
        // Adds a single task 
        public void add_task(string task, string notes, int64 date, int task_list) {
            this.db.prepare_v2(@"insert into ergo VALUES('$task', '$notes', '$date', 0, 0, '$task_list')", -1, out this.stmt, null);
            this.stmt.step();
            // update_task_list();
        }
        
        // Updates a single task 
        public void update_task(int id, string col, string newval) {
            this.db.prepare_v2(@"update ergo set $col = '$newval' where ROWID = '$id'", -1, out this.stmt, null);
            this.stmt.step();
        }
       
        // Task done function
        public void task_done(int id, int done) {
            this.db.prepare_v2(@"update ergo set done = '$done' where ROWID = '$id'", -1, out this.stmt, null);
            this.stmt.step();
            // update_task_list();
        }
        //*/
        
        // Removes all done tasks
        public void remove_done(){
            this.db.prepare_v2(@"delete from ergo where done = '1'",-1, out this.stmt, null);
            this.stmt.step();
            //   update_task_list();
        }
        
        public void get_interval(int64 date_from, int64 date_to, int task_list){
            string sql = @"select ROWID,task,notes,date,done from ergo where task_list_id = '$task_list'";
            if (date_from > 0) { sql += @" and date >= '$date_from'";}
            if (date_to > 0) { sql += @" and date <= '$date_to'";}
            
            this.db.prepare_v2(sql, -1,out this.stmt, null);
            int is_row = 0;
            do {
                is_row = stmt.step();
                if (is_row == Sqlite.ROW)
                {
                    var id = stmt.column_int(0);
                    var task = stmt.column_text(1);
                    var notes = stmt.column_text(2);
                    var date = stmt.column_int(3);
                    var done = stmt.column_int(4);
                   //print ("%d, %s, %s, %d, %d", id, task, notes, date, done);                    
                    this.window.tasks.add_task(id, task, notes, date, done);
                }
                
            } while (is_row == Sqlite.ROW);             
        }
        
        // Gets all tasks
        public void get_all_in_tasklist(int tasklist) {
            this.db.prepare_v2(@"select ROWID,task,notes,date,done from ergo where task_list_id = '$tasklist'",-1,out this.stmt, null);
            int is_row = 0;
            do {
                is_row = stmt.step();
                if (is_row == Sqlite.ROW)
                {
                    var id = stmt.column_int(0);
                    var task = stmt.column_text(1);
                    var notes = stmt.column_text(2);
                    var date = stmt.column_int(3);
                    var done = stmt.column_int(4);
//                    print ("%d, %s, %s, %s, %d", id, task, notes, date, done);
                    this.window.tasks.add_task(id, task, notes, date, done);
                }
                
            } while (is_row == Sqlite.ROW);   
        }
        
        // Gets all tasks
        public void get_tasklists() {
            this.db.prepare_v2(@"select ROWID,* from task_lists",-1,out this.stmt, null);
            int is_row = 0;
            do {
                is_row = stmt.step();
                if (is_row == Sqlite.ROW)
                {
                    var id = stmt.column_int(0);
                    var list_name = stmt.column_text(1);
                    var tasks = stmt.column_int(2);
                    var tasks_done = stmt.column_int(3);
//                    print ("%d, %s, %d, %d", id, list_name, tasks, tasks_done);
                    this.window.sidebar.lists.add_task_list(id, list_name, tasks, tasks_done);
                }
                
            } while (is_row == Sqlite.ROW);   
        }        
    }
}



