//  
//  Copyright (C) 2011 Ergo Developers
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using Gtk;
using Ergo;
using Ergo.Backend;

namespace Ergo.Widgets {

	public class ErgoTaskView : Gtk.TreeView {
	
	    public ListStore list_model { get; public set; }
    
        public void add_task( int id, string task,  string notes, int64 date, int done){
            
            var date_show = new DateTime.from_unix_utc(date);
            TreeIter iter;
            list_model.append(out iter); 
            list_model.set(iter, 0, id, 1, task, 2, notes, 3, date_show.format("%e.%B %Y"), 4 ,done);
        }
        
        public void show_folder(string label){

            int64 date_from = 0;
            int64 date_to = 0;
            var one_day = 86400;
            var today_date = new DateTime.now_local();
            int64 today = today_date.to_unix();
            today -= today % one_day;
            
            var days_left_in_week = 7 - today_date.get_day_of_week();
            print("%d", days_left_in_week);
                   
            switch (label) {
                case "Today":
                    date_from = today;
                    date_to = today;
                    break;
                case "Tomorrow":
                    date_from = today + one_day;
                    date_to = today + one_day;
                    break;
                case "This week":
                    date_from = today;
                    date_to = today + one_day * days_left_in_week;
                    break;
                case "Later":
                    date_from = today + one_day * days_left_in_week; 
                    date_to = 0;
                    break;
                default: //ALL
                    date_from = 0;
                    date_to = 0;       
                    break;            
            }
            
            int task_list = 0; //For now
            this.clear_model();
            this.window.backend.get_interval(date_from, date_to, task_list);
        }
        
        public void clear_model(){       
            list_model.clear();
        }        

        public void update_model(){       
            list_model.clear();
            window.backend.get_all_in_tasklist(window.sidebar.lists.selected_tasklist);
        }

        private ErgoWindow window;
		public ErgoTaskView(ErgoWindow window) {
            
            this.window = window;
		    headers_visible = false;
		    //even_row_color = gtk.gdk.color_parse ( "#e9e9e9");
		    //odd_row_color = gtk.gdk.color_parse ( "#f3f3f3");
		

		    list_model = new ListStore(5,		typeof(int),     	    // ID
		                                        typeof(string),     	// Task name
												typeof(string),     	// Notes
												typeof(string),     	// Date due
												typeof(bool)   		// Done - Toggle												
            );												
		    set_model(list_model);

			var column = new Gtk.TreeViewColumn();
            
            // Done toggle column
            var toggle = new CellRendererToggle();
        	toggle.toggled.connect((toggle, path) => {
            	var tree_path = new TreePath.from_string(path);
		        TreeIter iter;
		        list_model.get_iter (out iter, tree_path);
		        list_model.set (iter, 4, !toggle.active);
		        
		        Value val;
		        list_model.get_value(iter, 0, out val);
		        int id = val.get_int();
		        list_model.get_value(iter, 4, out val);
		        bool done_t = val.get_boolean();
		        
		        var done = 0;
		        if (done_t) { done = 1;}		        
		        
                this.window.backend.task_done(id, done);
		        
        	});
        	
            column.pack_start (toggle, false);
        	column.add_attribute (toggle, "active", 4);
			insert_column(column, -1);
			
			// Task name cloumn
	        insert_column_with_attributes(-1, "Task", new CellRendererText(), "text", 1);
	        	        
	        // Expander column
	        column = new Gtk.TreeViewColumn();
            column.set_expand(true);
            insert_column(column, -1);
            
            // Date column
            insert_column_with_attributes(-1, "Date", new CellRendererText(), "text", 3);
            
            // Remove column
            column = new Gtk.TreeViewColumn();
            var remove_pixbuf = new CellRendererPixbuf();
            /*//Removes
            remove_pixbuf.clicked.connect((path) => {
                //Delete row from backend -> refresh model
                print("\nDelete something\n");
			});
			//*/                        
            remove_pixbuf.stock_id = "gtk-close";
            column.pack_start (remove_pixbuf, false);
			insert_column(column, -1);
		}
/*		
		private void setup_treeview(){
		    // Must set up the treeview
		}
*/	
	}
}

