//  
//  Copyright (C) 2011 Ergo Developers
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using Gtk;
using Gdk;

using Granite.Widgets;
using Granite.Services;

using Ergo;
using Ergo.Backend;

namespace Ergo.Widgets {

	public class ErgoWindow : Gtk.Window {
	
		private VBox vbox;
		public ErgoToolbar toolbar { get; private set; }
        public ErgoAddBar add_bar { get; private set; }
		public ErgoFolder folders { get; private set; }
		public ErgoTaskView tasks { get; private set; }
		public ErgoSideBar sidebar { get; private set; }		
		public ErgoSqlBackend backend {get; private set;}
        public static Granite.Application app { get; private set; }
                
		public ErgoWindow (Granite.Application app) {
			this.app = app;
			
            var mainbox = new VBox (false, 0);
            vbox = new VBox (false, 0);
            toolbar = new ErgoToolbar (this);
            folders = new ErgoFolder (this); 
            tasks = new ErgoTaskView (this);
            add_bar = new ErgoAddBar(this);
            sidebar = new ErgoSideBar(this);
            
		    // Backend
		    backend = new ErgoSqlBackend(this);
            backend.get_all_in_tasklist(0);
            backend.get_tasklists(); 


			// Window Properties
			title = "Ergo";
			icon_name = "ergo";
			set_size_request (430, 300);
			restore_saved_state ();	

			// Initialize layout		
			var scrolled_window = new ScrolledWindow (null, null);
			scrolled_window.set_policy (PolicyType.NEVER, PolicyType.AUTOMATIC);
			scrolled_window.set_shadow_type (ShadowType.NONE);
			var viewport = new Viewport (null, null);
			viewport.shadow_type = ShadowType.NONE;
			
			
			viewport.add (tasks);
			scrolled_window.add (viewport);
					
			mainbox.pack_start (toolbar, false, false, 0);
//			vbox.pack_start (toolbar, false, false, 0);		
            mainbox.pack_start (folders, false, false, 0);
//			vbox.pack_start (folders, false, false, 0);
			vbox.pack_start(add_bar, false, false, 1);
			vbox.pack_end (scrolled_window, true, true, 0);
			
			            // Sidebar comes in HPaned one side
            var hpane = new HBox(false, 0);
            hpane.pack_start(vbox, true, true, 0);
            hpane.pack_start(sidebar, false, false, 0);
            
            mainbox.pack_start (hpane, true, true, 0);
			add (mainbox);
			
			// Signals and callbacks
			destroy.connect (Gtk.main_quit);
		}

		protected override bool delete_event (Gdk.EventAny event) {
			update_saved_state ();
			return false;
		}
		
		private void restore_saved_state () {
			
			// Restore window state
			default_width = ErgoApp.saved_state.window_width;
			default_height = ErgoApp.saved_state.window_height;
			
			if (ErgoApp.saved_state.window_state == ErgoWindowState.MAXIMIZED)
				maximize ();
			else if (ErgoApp.saved_state.window_state == ErgoWindowState.FULLSCREEN)
				fullscreen ();
			
		}
		
		private void update_saved_state () { 
		//last open folder will be saved aswell
			
			// Save window state
			if ((get_window ().get_state () & WindowState.MAXIMIZED) != 0)
				ErgoApp.saved_state.window_state = ErgoWindowState.MAXIMIZED;
			else if ((get_window ().get_state () & WindowState.FULLSCREEN) != 0)
				ErgoApp.saved_state.window_state = ErgoWindowState.FULLSCREEN;
			else
				ErgoApp.saved_state.window_state = ErgoWindowState.NORMAL;
			
			// Save window size
			if (ErgoApp.saved_state.window_state == ErgoWindowState.NORMAL) {
				int width, height;
				get_size (out width, out height);
				ErgoApp.saved_state.window_width = width;
				ErgoApp.saved_state.window_height = height;
			}
			
		}
		
	}
	
}
